/* PCF2129AT Arduino用ライブラリ ver 0.7
 * memo:
 * 11/5 22.30 時計機能追加
 * 11/16 00:00
 * 11/30 23:00 アラーム機能追加
 *
 * NXPより公開されているmbed向けlibraryおよび
 * CQ出版より公開されているサンプルコードを
 * 参考にしています.
*/

#include <Arduino.h>
#include <Wire.h>
#include "PCF2129AT.h"

int cmd[8];//送受信用のキャッシュ


/*初期化*/
void PCF2129AT::init()
{
  //デバイスの仕様としては
  //24時間モードか12時間モードの設定が出来たり
  //アラームの設定が出来たり
  //外部に割り込みの出力が出来たり
  //etc....

}

/*日時の取得*/
void  PCF2129AT::getDate(dt_dat *dt)
{
  PCF2129AT::dataRead(PCF2129AT_ADDR,  Seconds, cmd, 7);
  // if(cmd[0]&=0x80) return -1;//時刻設定ができていない可能性(起動時など)
  cmd[0] &= 0x7f;                         // 有効なのは下位7ビット
  dt->s = (cmd[0] >> 4) * 10 + (cmd[0] & 0xf);    // BCDの数値化

  cmd[1] &= 0x7f;                         // 有効なのは下位7ビット
  dt->m = (cmd[1] >> 4) * 10 + (cmd[1] & 0xf);    // BCDの数値化

  cmd[2] &= 0x3f;                         // 有効なのは下位6ビット
  dt->h = (cmd[2] >> 4) * 10 + (cmd[2] & 0xf);    // BCDの数値化

  cmd[3] &= 0x3f;                         // 有効なのは下位6ビット
  dt->d = (cmd[3] >> 4) * 10 + (cmd[3] & 0xf);    // BCDの数値化

  dt->wd = cmd[4];                        // BCDの数値化

  cmd[5] &= 0x1f;                         // 有効なのは下位5ビット
  dt->mm = (cmd[5] >> 4) * 10 + (cmd[5] & 0xf);   // BCDの数値化

  dt->y = (cmd[6] >> 4) * 10 + (cmd[6] & 0xf);    // BCDの数値化

}


/*日時の設定*/
void PCF2129AT::setDate(dt_dat *dt)
{
    cmd[0] = ((dt->s / 10) << 4) + (dt->s % 10); // 秒のBCD化
    cmd[1] = ((dt->m / 10) << 4) + (dt->m % 10);        // 分のBCD化
    cmd[2] = ((dt->h / 10) << 4) + (dt->h % 10);        // 時のBCD化
    cmd[3] = ((dt->d / 10) << 4) + (dt->d % 10);        // 日のBCD化
    cmd[4] = dt->wd;
    cmd[5] = ((dt->mm / 10) << 4) + (dt->mm % 10);      // 月のBCD化
    dt->y = dt->y - 2000;
    cmd[6] = ((dt->y / 10) << 4) + (dt->y % 10);        // 年のBCD化
    PCF2129AT::dataSend(PCF2129AT_ADDR,  Seconds, cmd, 7);      // 日時の設定
}

/*秒をゼロに戻す*/
// プラスマイナス30秒未満のズレなら簡単に修正できる
void PCF2129AT::setSecondToZero()
{
    PCF2129AT::dataRead(PCF2129AT_ADDR,  Seconds, cmd, 7);
    cmd[0] = 0x80;
    PCF2129AT::dataSend(PCF2129AT_ADDR,  Seconds, cmd, 7);
}

void PCF2129AT::setAlarm(dt_dat *dt)
{
    PCF2129AT::clearAF();
    /* アラーム時刻の設定 */
    cmd[0] = ((dt->s / 10) << 4) + (dt->s % 10) + ((_AlarmDisable_Second)<<7); // 秒のBCD化
    cmd[1] = ((dt->m / 10) << 4) + (dt->m % 10) + ((_AlarmDisable_Minute)<<7);        // 分のBCD化
    cmd[2] = ((dt->h / 10) << 4) + (dt->h % 10) + ((_AlarmDisable_Hour)<<7);        // 時のBCD化
    cmd[3] = ((dt->d / 10) << 4) + (dt->d % 10) + ((_AlarmDisable_Day)<<7);        // 日のBCD化
    cmd[4] = dt->wd + ((_AlarmDisable_WeekDay)<<7);
    cmd[5] = ((dt->mm / 10) << 4) + (dt->mm % 10);      // 月のBCD化
    dt->y = dt->y - 2000;
    cmd[6] = ((dt->y / 10) << 4) + (dt->y % 10);        // 年のBCD化

    /* アラームの判定に使うデータの設定 /
    cmd[0] |= (!_AlarmDisable_Second)<<7; // 7bit目
    cmd[1] |= (!_AlarmDisable_Minute)<<7; // 7bit目
    cmd[2] |= (!_AlarmDisable_Hour)<<7; // 7bit目
    cmd[3] |= (!_AlarmDisable_Day)<<7; // 7bit目
    cmd[4] |= (!_AlarmDisable_WeekDay)<<7; // 7bit目
*/
    PCF2129AT::dataSend(PCF2129AT_ADDR,  Second_alarm, cmd, 7);      // 時刻設定と判定の書き込み
}

void PCF2129AT::enableInterruptAlarm()
{
  _InterruptEnableAlarm = true;
}
void PCF2129AT::enableInterruptTimeStamp()
{
  _InterruptEnableTimeSt = true;
}
void PCF2129AT::enableInterruptSecMin()
{
  _InterruptEnableSecMin = true;
}
void PCF2129AT::enableInterruptWatchDog()
{
  _InterruptEnableWatchDG = true;
}
void PCF2129AT::enableInterruptBattery()
{
  _InterruptEnableBat = true;
}
void PCF2129AT::enableInterruptLowBattery()
{
  _InterruptEnableLowBat = true;
}

void PCF2129AT::setInterruptsPIN()
{
  PCF2129AT::dataRead(PCF2129AT_ADDR,  Control_2, cmd, 2);
  if(_InterruptEnableAlarm)
  {
    cmd[0] |= AIE;
    //Serial.println("AIE has enabled");
  }
  if(_InterruptEnableTimeSt)
  {
    cmd[0] |= TSIE;
    //Serial.println("TSIE has enabled");
  }
  if(_InterruptEnableSecMin){
    cmd[1] |= SI_MI;
    //Serial.println("SIMI has enabled");
  }
  if(_InterruptEnableWatchDG)
  {
    cmd[1] |= WD_CD;
    //Serial.println("WDCD has enabled");
  }
  if(_InterruptEnableBat){
    cmd[1] |= BIE;
    //Serial.println("BIE Writte");
  }
  if(_InterruptEnableLowBat)
  {
    cmd[1] |= BLIE;
    //Serial.println("BLIE has enabled");
  }
  PCF2129AT::dataSend(PCF2129AT_ADDR,  Control_2, cmd, 2);
}

void PCF2129AT::enableAlarmSecond()
{
  _AlarmDisable_Second = false;
}

void PCF2129AT::enableAlarmMinute()
{
  _AlarmDisable_Minute = false;
}

void PCF2129AT::enableAlarmHour()
{
  _AlarmDisable_Hour = false;
}

void PCF2129AT::enableAlarmDay()
{
  _AlarmDisable_Day = false;
}

void PCF2129AT::enableAlarmWeekDay()
{
  _AlarmDisable_WeekDay = false;
}


void PCF2129AT::clearAF()
{
  PCF2129AT::dataRead(PCF2129AT_ADDR,  Control_2, cmd, 1);
  cmd[0] ^= AFlag;
  PCF2129AT::dataSend(PCF2129AT_ADDR,  Control_2, cmd, 1);
}

void PCF2129AT::clearBLFlag()
{
  PCF2129AT::dataRead(PCF2129AT_ADDR,  Control_3, cmd, 1);
  cmd[0] ^= BLFlag;
  PCF2129AT::dataSend(PCF2129AT_ADDR,  Control_3, cmd, 1);
}


//I2C通信で複数受信
//https://code.google.com/p/arduino/issues/detail?id=28
/*I2Cで配列のデータを送る*/
/*送信先アドレス,送信先レジスタ,送信する配列データ,送信する配列要素数*/
void PCF2129AT::dataSend(int id,int reg,int *data,int datasize)
{
  Wire.beginTransmission(id);
  Wire.write(reg);
  for(int i=0;i<datasize;i++)
  {
    Wire.write(data[i]);
    //Serial.print("data sended : "); //debug
    //Serial.println(data[i], BIN); //debug
  }
  Wire.endTransmission();
}

/*I2Cで配列のデータを受け取る*/
/*受信元アドレス,受信元レジスタ,受信する配列データ,受信する配列要素数*/
void PCF2129AT::dataRead(int id,int reg,int *data,int datasize)
{
  //Serial.print("transmission begin");//debug
  Wire.beginTransmission(id);             //スレーブアドレス送信,書き込みモード
  Wire.write(reg);                        //レジスタアドレス送信
  Wire.endTransmission(true);            //STOPコンディションを送る
  //STOPコンディションを送る仕様
  Wire.requestFrom(id, datasize, false);  //スレーブアドレス送信,受信モード,
  for(int i=0;i<datasize;i++)
  {
    data[i] = Wire.read();
    //Serial.print("data recived : ");//debug
    //Serial.println(data[i], BIN);//debug
  }
  Wire.endTransmission(true);
  //Serial.println("transmission end");//debug
}
