
#ifndef PCF2129AT_h
#define PCF2129AT_H

#define Control_1        0x00
#define Control_2        0x01
#define Control_3        0x02
#define Seconds          0x03
#define Minutes          0x04
#define Hours            0x05
#define Days             0x06
#define Weekdays         0x07
#define Months           0x08
#define Years            0x09
#define Second_alarm     0x0a
#define Minute_alarm     0x0b
#define Hour_alarm       0x0c
#define Day_alarm        0x0d
#define Weekday_alarm    0x0e
#define CLKOUT_ctl       0x0f
#define Watchdg_tim_ctl  0x10
#define Watchdg_tim_val  0x11
#define Timestp_ctl      0x12
#define Sec_timestp      0x13
#define Min_timestp      0x14
#define Hour_timestp     0x15
#define Day_timestp      0x16
#define Mon_timestp      0x17
#define Year_timestp     0x18
#define Aging_offset     0x19
#define Internal_reg     0x1a

/* INTピンの許可設定 */
//reg2
#define AIE 0x02// アラーム割り込み許可設定
#define TSIE 0x03// タイムスタンプ割り込み許可設定
//reg3
#define SI_MI 0x80// 秒or分ごとの割り込み許可設定
#define WD_CD 0x40// ウオッチドッグタイマの割り込み
#define BIE 0x02// バッテリ動作割り込み許可設定
#define BLIE 0x01// バッテリ低残量割り込み許可設定

/* 割り込みフラグ */
//reg2
#define AFlag 0x10;
#define WDTFlag 0x40;
#define MSFlag 0x80;
//reg4
#define BLFlag 0x04;
#define BFlag 0x08;

#define PCF2129AT_ADDR   0x51

typedef struct
{
  char   s;         // 秒
  char   m;         // 分
  char   h;         // 時間
  char   d;         // 日
  char   wd;        // 曜日
  char   mm;        // 月
  short   y;        // 年
} dt_dat, *pdt_dat;




class PCF2129AT
{
  public:
    PCF2129AT(){};
    void setDate(dt_dat *);
    void getDate(dt_dat *);

    void setSecondToZero();

    // INTピンに反映される割り込みの許可設定
    void enableInterruptAlarm();
    void enableInterruptTimeStamp();
    void enableInterruptSecMin();
    void enableInterruptWatchDog();
    void enableInterruptBattery();
    void enableInterruptLowBattery();
    void setInterruptsPIN(); // 書き込み

    // アラームの判定機能の有効化
    void enableAlarmSecond();
    void enableAlarmMinute();
    void enableAlarmHour();
    void enableAlarmDay();
    void enableAlarmWeekDay();
    void setAlarm(dt_dat *dt); // 書き込み

    void clearAF();
    void clearBLFlag();

    //中身は空
    void init();

    /* 各割り込みフラグの取得(未実装)
    // 即時性を求めない場合などに
    bool getFlagWD_CD(){};
    bool getFlagMSF(){};
    bool getFlagAF(){};
    bool getFlagTSFx(){};
    bool getFlagBF(){}:
    bool getFlagBLF(){};
    */
    ~PCF2129AT(){};

    void dataSend(int,int,int*,int);
    void dataRead(int,int,int*,int);
  private:
    // interrupts elable config
    // defuault = o (disable)
    bool _InterruptEnableAlarm=0;
    bool _InterruptEnableTimeSt=0;
    bool _InterruptEnableSecMin=0;
    bool _InterruptEnableWatchDG=0;
    bool _InterruptEnableBat=0;
    bool _InterruptEnableLowBat=0;


    // alerm check disable
    // default = 1(disable)
    bool _AlarmDisable_Second = 1;
    bool _AlarmDisable_Minute = 1;
    bool _AlarmDisable_Hour = 1;
    bool _AlarmDisable_Day = 1;
    bool _AlarmDisable_WeekDay =1;

};

#endif
