/*
 *  I2Cで接続したPCF2129ATに
 *  日時とアラームの時刻を書き込み
 *  アラームの判定を割り込みピンで検知するプログラム
 *  Arduinoの2番ピンとPCF2129ATのINTピンを接続してください
 *  通信はI2Cです
 *  http://trac.switch-science.com/wiki/PCF2129AT
 */
#include <Wire.h>
#include "PCF2129AT.h"

PCF2129AT rtc;
dt_dat dt;
int debug[0x19];

bool AlarmDetected = false;
void setup()
{
    pinMode(2,INPUT_PULLUP);// 割り込み用ピンの設定,内部プルアップ有効(~INTピンはオープンドレイン)
    pinMode(3,INPUT_PULLUP);// ボタン

    Serial.begin(9600);
    Wire.begin();
    rtc.init();
    Serial.println("PC2129AT Sample Program");


    /* 現在時刻の設定 */
    dt.y = 2015;                // 年の設定(2000年代)
    dt.wd = 5;                  // 曜日の設定(日:0-土:6)
    dt.mm = 11;                 // 月の設定(1-12)
    dt.d = 27;                   // 日の設定(1-)
    dt.h = 21;                   // 時の設定
    dt.m = 56;                  // 分の設定
    dt.s = 30;                   // 秒の設定
    rtc.setDate(&dt);              // 日時の設定

    /* アラーム日時の設定 */
    dt_dat dt_alarm;
    dt_alarm.wd = 5;                  // 曜日の設定(日:0-土:6)
    dt_alarm.d = 27;                   // 日の設定(1-)
    dt_alarm.h = 21;                   // 時の設定
    dt_alarm.m = 56;                  // 分の設定
    dt_alarm.s = 45;                   // 秒の設定

    /* 有効にするアラームの設定 */
    // デフォルト:無効
    rtc.enableAlarmSecond(); // 秒の一致を有効
    //rtc.enableAlarmMinute(); // 分の一致を有効
    //rtc.enableAlarmHour(); // 時の一致を有効
    //rtc.enableAlarmDay(); // 日にちの一致を有効
    //rtc.enableAlarmWeekDay(); // 曜日の一致を有効
    // 最後にsetAlarmをしてくだい(一致判定もこの時書き込まれます)
    rtc.setAlarm(&dt_alarm); // アラームの設定時刻の書き込み

    /* 有効にする割り込み許可の設定 */
    // 基本的にINTピンには各割り込みの論理和が適用されます
    // 利用したい割り込み機能だけ呼んでください
    rtc.enableInterruptAlarm();
    //rtc.enableInterruptTimeStamp();
    //rtc.enableInterruptSecMin();
    //rtc.enableInterruptWatchDog();
    //rtc.enableInterruptBattery();
    //rtc.enableInterruptLowBattery();
    // 設定後に書き込みます
    rtc.setInterruptsPIN();

    rtc.clearAF(); // アラームの割り込みフラグをクリア(初期化9
    //rtc.clearBLFlag();


    /* 外部ピン割り込みの設定 */
    attachInterrupt(0, detectAlarm, FALLING ); // デジタルピン2で検知
    attachInterrupt(1, clearAlarm, FALLING ); // デジタルピン3で検知}

void loop() {

  if(AlarmDetected)
  {
    Serial.println("Alarm RINGINGGGGG!!!!!!!!");
    rtc.getDate(&dt);          // 日時の取得
    // 日時の表示
    Serial.print("year :");
    Serial.print(2000 + dt.y,DEC);
    Serial.print("\t week :");
    Serial.print(dt.wd,DEC);
    Serial.print("\t month :");
    Serial.print(dt.mm,DEC);
    Serial.print("\t date :");
    Serial.print(dt.d,DEC);
    Serial.print("\t hour :");
    Serial.print(dt.h,DEC);
    Serial.print("\t minutes :");
    Serial.print(dt.m,DEC);
    Serial.print("\t second :");
    Serial.print(dt.s,DEC);
    Serial.print("\t");
    Serial.println(digitalRead(2));
    rtc.clearAF();
    AlarmDetected = false;
  }
  rtc.getDate(&dt);          // 日時の取得
  // 日時の表示
  Serial.print("year :");
  Serial.print(2000 + dt.y,DEC);
  Serial.print("\t week :");
  Serial.print(dt.wd,DEC);
  Serial.print("\t month :");
  Serial.print(dt.mm,DEC);
  Serial.print("\t date :");
  Serial.print(dt.d,DEC);
  Serial.print("\t hour :");
  Serial.print(dt.h,DEC);
  Serial.print("\t minutes :");
  Serial.print(dt.m,DEC);
  Serial.print("\t second :");
  Serial.print(dt.s,DEC);
  Serial.print("\t");
  Serial.println(analogRead(2));




  delay(50);
}

/* 立ち下がりを検知次第実行される */
void detectAlarm()
{
  AlarmDetected = true;
  // Arduinoの仕様上ここでWireインスタンスを利用できない
  // (割り込みの許可不許可が入れ込みハングアップする)
}
